/*
 * Disables RSA signature check
 */

#include <dlfcn.h>
#include <stdio.h>
#include <sys/mman.h>

#define NUM_PATCHES	2

static void cacheflush(void *start, size_t size)
{
   /* For details, please see the Linux sources, files
    * arch/arm/kernel/traps.c, arch/arm/kernel/entry-common.S and
    * asm-arm/unistd.h
    *
    * This syscall is supported by all 2.4 and 2.6 Linux kernels.  2.2 linux
    * kernels got first support with version 2.2.18.
    */

   asm("mov r0, %0\n"
       "mov r1, %1\n"
       "mov r2, #0\n"
       /* EABI or Thumb syscall: syscall number passed in 'r7' (syscall base
	* number is 0x0).  Note that Thumb and EABI are generally not the
	* same.  It just happens that the simple cacheflush syscall doesn't
	* expose any of differences in calling conventions.
        */
       "mov r7, #0xf0000\n"
       "add r7, #0x02\n"
       "swi #0\n" ::
       "g"(start), "g"((long)start+size) :
       "r0", "r1", "r2", "r7");
}

int Game_Main(const char  *path, const char *udn __attribute__ ((unused)))
{
	void *myself;
	unsigned *patch;
	unsigned pageaddr;
	FILE *msg;

	/* Open log */
	msg = fopen("/dev/kmsg","a+");

	/* open exeDSP */
	if ( (myself = dlopen(NULL,RTLD_NOW | RTLD_GLOBAL)) == NULL )
	{
		fprintf(msg,"Cannot dlopen myself.\n");
		return 1;
	}

	/* Get address of RSA_verify routine */
	if ( (patch = dlsym(myself,"RSA_verify")) == NULL )
	{
		fprintf(msg,"Cannot find address of RSA_verify.\n");
		dlclose(myself);
		return 1;
	}

	dlclose(myself);

	/* Remove write protection */
	pageaddr = ((unsigned) patch) & ~0x0FFF;
	mprotect((void *) pageaddr, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);

	/* Patch routine */
	patch[0] = 0xE3A00001;	/* MOV R0, #1 */
	patch[1] = 0xE1A0F00E;	/* MOV PC, LR */

	/* Set write protection */
	mprotect((void *) pageaddr, 8192, PROT_READ | PROT_EXEC);

	/* Flush cache */
	cacheflush(patch, 2);

	fprintf(msg,"RSA_verify at %p patched.\n",patch);

	fclose(msg);

	return 1;
}
